# -*- coding: utf-8 -*-
"""

 This file is part of oe3d (www.oe3d.at).

 oe3d is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 oe3d is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with oe3d.  If not, see <http://www.gnu.org/licenses/>.

 Author(s): Gerhard Summer (gerhard.summer@rechenraum.com) 
            Simon Flöry (simon.floery@rechenraum.com)
            

"""

import ConfigParser
import oe3d
import numpy as np
import time
import shutil
import os
import neighbors
from collections import namedtuple
from multiprocessing import Pool

Config = ConfigParser.ConfigParser()
Config.read('./configfile.txt')

A=Config.getint("compute_all", "nr_subkacheln")
lam=Config.getfloat("compute_all", "lam")
Ol=Config.getint("compute_all", "Overlap")
path=Config.get("dem","resultatpfad")
objexportpfad = Config.get("dem", "objexportpfad")
nrProcesses = int(Config.get("compute_all", "nrProcesses"))


X=[9,10,11,12,13,14,15,16,9,10,11,12,13,14,15,16,17,12,13,14,15,16,17,14,15]
Y=[46,46,46,46,46,46,46,46,47,47,47,47,47,47,47,47,47,48,48,48,48,48,48,49,49]

#X=[9]
#Y=[46]

################################################################################

def computeTile(tileInfo):
    """
    Computes one sub tile. We need to put this into a function to be able
    to call it in parallel.
    
    tileInfo Triple (i,j,k) of values. i is the index into X and Y. j is the
        row index of the sub tile and k the column index.
    """
    
    xi = X[tileInfo[0]];
    yi = Y[tileInfo[0]];
    j = tileInfo[1];
    k = tileInfo[2];
    
    newpath='0raw'
    name='N'+str(yi)+'E'+(str(xi)).zfill(3)+'_'+str(j)+str(k)+'.tif'            
    
    # Testing for existance gives problems when called in parallel. So simply
    # catch the error for an existing directory.
    try:
        os.makedirs(path+newpath)
    except:
        # That's a python nop 
        pass
        
    if k==0:
        dx=(-6400.-Ol)/10000
    else:
        dx=-Ol/10000.
    llu=xi+k*3600./(A*10000)+dx
    
    if j==A-1:
        dy=(-6400.-Ol)/10000.
    else:
        dy=-Ol/10000.
    blu=yi+(A-1-j)*3600./(A*10000)+dy
    
    print 'Compute sub-patch ' + name
    oe3d.compute(llu,blu,3600./(A*10000)+2*Ol/10000.,3600./(A*10000)+2*Ol/10000.,lam,newpath+'/'+name)
    

################################################################################

def compute():

    
    startZeit = time.time()
    if True:
        if (objexportpfad != ''):
            print 'objexportpfad must not be set when using compute_all.'
            exit()
    
        if (3600./A)-np.trunc(3600./A)!=0:
            print('A is no divider of 3600!')
            exit()
    
        tileInfo = [];
        for i in range(len(X)):
            for j in range(A):
                for k in range(A):
                    tileInfo.append([i, j, k]);
        pool = Pool(processes=nrProcesses)
        pool.map(computeTile, tileInfo)
        #computeTile(tileInfo[0])
                    
        newpath='0raw'
        resultatpfad=path+newpath
        resultatpfadneu=path+'1blended'
        if not os.path.exists(resultatpfadneu):
            os.makedirs(resultatpfadneu)
        (rummax,cnummax)=neighbors.neighbors(resultatpfad,resultatpfadneu)
            
        reslist=os.listdir(resultatpfadneu)
        for resfile in reslist:
            shutil.move(resultatpfadneu+'/'+resfile, resultatpfadneu+'/'+resfile[0:7]+'/')
    
        if not os.path.exists(path+'2merged'):
                os.makedirs(path+'2merged')
        for i in range(len(X)):
            os.system("gdal_merge.py -of GTiff -o "+path+"2merged/N"+str(Y[i])+"E"+str(X[i]).zfill(3)+".tif "+path+"1blended/N"+str(Y[i])+"E"+str(X[i]).zfill(3)+"/*.tif")
    
    if not os.path.exists(path+'3clipped'):
        os.makedirs(path+'3clipped')
    for i in range(len(X)):
        data=oe3d.fileloader(path+'2merged/N'+str(Y[i])+'E'+str(X[i]).zfill(3)+'.tif')
        datacut=oe3d.haircut(X[i], Y[i], 1, 1, data)
        oe3d.fileexporter(X[i],Y[i],1,1,datacut[:,2],path+'3clipped/N'+str(Y[i])+'E'+str(X[i]).zfill(3)+'.tif')
    return;
    
    
    print 'Vergangene Zeit (gesamt): ' + str(time.time() - startZeit)
    
    

################################################################################

compute();

